"""MTA Subway Lines Enums and Constants

This file contains enums and mappings for subway lines to the GTFS Stop IDs they
service.
Generated from MTA_Subway_Stations_20250702.csv
"""

from enum import Enum
from typing import Dict, List


class SubwayLine(Enum):
    """Enum for all NYC subway lines"""

    ONE = "1"
    TWO = "2"
    THREE = "3"
    FOUR = "4"
    FIVE = "5"
    SIX = "6"
    SEVEN = "7"
    A = "A"
    B = "B"
    C = "C"
    D = "D"
    E = "E"
    F = "F"
    G = "G"
    J = "J"
    L = "L"
    M = "M"
    N = "N"
    Q = "Q"
    R = "R"
    S = "S"
    SIR = "SIR"  # Staten Island Railway
    W = "W"
    Z = "Z"


# Mapping of subway lines to the stops they service
SUBWAY_LINES: Dict[str, List[str]] = {
    "1": [
        "101",
        "103",
        "104",
        "106",
        "107",
        "108",
        "109",
        "110",
        "111",
        "112",
        "113",
        "114",
        "115",
        "116",
        "117",
        "118",
        "119",
        "120",
        "121",
        "122",
        "123",
        "124",
        "125",
        "126",
        "127",
        "128",
        "129",
        "130",
        "131",
        "132",
        "133",
        "134",
        "135",
        "136",
        "137",
        "138",
        "139",
        "142",
    ],
    "2": [
        "120",
        "123",
        "127",
        "128",
        "132",
        "137",
        "201",
        "204",
        "205",
        "206",
        "207",
        "208",
        "209",
        "210",
        "211",
        "212",
        "213",
        "214",
        "215",
        "216",
        "217",
        "218",
        "219",
        "220",
        "221",
        "222",
        "224",
        "225",
        "226",
        "227",
        "228",
        "229",
        "230",
        "231",
        "232",
        "233",
        "234",
        "235",
        "236",
        "237",
        "238",
        "239",
        "241",
        "242",
        "243",
        "244",
        "245",
        "246",
        "247",
    ],
    "3": [
        "120",
        "123",
        "127",
        "128",
        "132",
        "137",
        "224",
        "225",
        "226",
        "227",
        "228",
        "229",
        "230",
        "231",
        "232",
        "233",
        "234",
        "235",
        "236",
        "237",
        "238",
        "239",
        "248",
        "249",
        "250",
        "251",
        "252",
        "253",
        "254",
        "255",
        "256",
        "257",
        "301",
        "302",
    ],
    "4": [
        "234",
        "235",
        "239",
        "250",
        "401",
        "402",
        "405",
        "406",
        "407",
        "408",
        "409",
        "410",
        "411",
        "412",
        "413",
        "414",
        "415",
        "416",
        "418",
        "419",
        "420",
        "423",
        "621",
        "626",
        "629",
        "631",
        "635",
        "640",
    ],
    "5": [
        "204",
        "205",
        "206",
        "207",
        "208",
        "209",
        "210",
        "211",
        "212",
        "213",
        "214",
        "215",
        "216",
        "217",
        "218",
        "219",
        "220",
        "221",
        "222",
        "234",
        "235",
        "239",
        "241",
        "242",
        "243",
        "244",
        "245",
        "246",
        "247",
        "416",
        "418",
        "419",
        "420",
        "423",
        "501",
        "502",
        "503",
        "504",
        "505",
        "621",
        "626",
        "629",
        "631",
        "635",
        "640",
    ],
    "6": [
        "601",
        "602",
        "603",
        "604",
        "606",
        "607",
        "608",
        "609",
        "610",
        "611",
        "612",
        "613",
        "614",
        "615",
        "616",
        "617",
        "618",
        "619",
        "621",
        "622",
        "623",
        "624",
        "625",
        "626",
        "627",
        "628",
        "629",
        "630",
        "631",
        "632",
        "633",
        "634",
        "635",
        "636",
        "637",
        "638",
        "639",
        "640",
    ],
    "7": [
        "701",
        "702",
        "705",
        "706",
        "707",
        "708",
        "709",
        "710",
        "711",
        "712",
        "713",
        "714",
        "715",
        "716",
        "718",
        "719",
        "720",
        "721",
        "723",
        "724",
        "725",
        "726",
    ],
    "A": [
        "A02",
        "A03",
        "A05",
        "A06",
        "A07",
        "A09",
        "A12",
        "A15",
        "A24",
        "A27",
        "A28",
        "A31",
        "A32",
        "A34",
        "A36",
        "A38",
        "A40",
        "A41",
        "A42",
        "A46",
        "A48",
        "A51",
        "A55",
        "A57",
        "A59",
        "A60",
        "A61",
        "A63",
        "A64",
        "A65",
        "H01",
        "H02",
        "H03",
        "H04",
        "H06",
        "H07",
        "H08",
        "H09",
        "H10",
        "H11",
        "H12",
        "H13",
        "H14",
        "H15",
    ],
    "B": [
        "A14",
        "A15",
        "A16",
        "A17",
        "A18",
        "A19",
        "A20",
        "A21",
        "A22",
        "A24",
        "D03",
        "D04",
        "D05",
        "D06",
        "D07",
        "D08",
        "D09",
        "D10",
        "D11",
        "D12",
        "D13",
        "D14",
        "D15",
        "D16",
        "D17",
        "D20",
        "D21",
        "D22",
        "D24",
        "D25",
        "D26",
        "D28",
        "D31",
        "D35",
        "D39",
        "D40",
        "R30",
    ],
    "C": [
        "A09",
        "A10",
        "A11",
        "A12",
        "A14",
        "A15",
        "A16",
        "A17",
        "A18",
        "A19",
        "A20",
        "A21",
        "A22",
        "A24",
        "A25",
        "A27",
        "A28",
        "A30",
        "A31",
        "A32",
        "A33",
        "A34",
        "A36",
        "A38",
        "A40",
        "A41",
        "A42",
        "A43",
        "A44",
        "A45",
        "A46",
        "A47",
        "A48",
        "A49",
        "A50",
        "A51",
        "A52",
        "A53",
        "A54",
        "A55",
    ],
    "D": [
        "A15",
        "A24",
        "B12",
        "B13",
        "B14",
        "B15",
        "B16",
        "B17",
        "B18",
        "B19",
        "B20",
        "B21",
        "B22",
        "B23",
        "D01",
        "D03",
        "D04",
        "D05",
        "D06",
        "D07",
        "D08",
        "D09",
        "D10",
        "D11",
        "D12",
        "D13",
        "D14",
        "D15",
        "D16",
        "D17",
        "D20",
        "D21",
        "D22",
        "D43",
        "R31",
        "R36",
    ],
    "E": [
        "A25",
        "A27",
        "A28",
        "A30",
        "A31",
        "A32",
        "A33",
        "A34",
        "D14",
        "E01",
        "F05",
        "F06",
        "F07",
        "F09",
        "F11",
        "F12",
        "G05",
        "G06",
        "G07",
        "G08",
        "G14",
        "G21",
    ],
    "F": [
        "A41",
        "B04",
        "B06",
        "B08",
        "B10",
        "D15",
        "D16",
        "D17",
        "D18",
        "D19",
        "D20",
        "D21",
        "D42",
        "D43",
        "F01",
        "F02",
        "F03",
        "F04",
        "F05",
        "F06",
        "F07",
        "F14",
        "F15",
        "F16",
        "F18",
        "F20",
        "F21",
        "F22",
        "F23",
        "F24",
        "F25",
        "F26",
        "F27",
        "F29",
        "F30",
        "F31",
        "F32",
        "F33",
        "F34",
        "F35",
        "F36",
        "F38",
        "F39",
        "G08",
        "G14",
    ],
    "G": [
        "A42",
        "F20",
        "F21",
        "F22",
        "F23",
        "F24",
        "F25",
        "F26",
        "F27",
        "G22",
        "G24",
        "G26",
        "G28",
        "G29",
        "G30",
        "G31",
        "G32",
        "G33",
        "G34",
        "G35",
        "G36",
    ],
    "J": [
        "G05",
        "G06",
        "J12",
        "J13",
        "J14",
        "J15",
        "J16",
        "J17",
        "J19",
        "J20",
        "J21",
        "J22",
        "J23",
        "J24",
        "J27",
        "J28",
        "J29",
        "J30",
        "J31",
        "M11",
        "M12",
        "M13",
        "M14",
        "M16",
        "M18",
        "M19",
        "M20",
        "M21",
        "M22",
        "M23",
    ],
    "L": [
        "L01",
        "L02",
        "L03",
        "L05",
        "L06",
        "L08",
        "L10",
        "L11",
        "L12",
        "L13",
        "L14",
        "L15",
        "L16",
        "L17",
        "L19",
        "L20",
        "L21",
        "L22",
        "L24",
        "L25",
        "L26",
        "L27",
        "L28",
        "L29",
    ],
    "M": [
        "D15",
        "D16",
        "D17",
        "D18",
        "D19",
        "D20",
        "D21",
        "F09",
        "F11",
        "F12",
        "G08",
        "G09",
        "G10",
        "G11",
        "G12",
        "G13",
        "G14",
        "G15",
        "G16",
        "G18",
        "G19",
        "G20",
        "G21",
        "M01",
        "M04",
        "M05",
        "M06",
        "M08",
        "M09",
        "M10",
        "M11",
        "M12",
        "M13",
        "M14",
        "M16",
        "M18",
    ],
    "N": [
        "D43",
        "N02",
        "N03",
        "N04",
        "N05",
        "N06",
        "N07",
        "N08",
        "N09",
        "N10",
        "Q01",
        "R01",
        "R03",
        "R04",
        "R05",
        "R06",
        "R08",
        "R09",
        "R11",
        "R13",
        "R14",
        "R15",
        "R16",
        "R17",
        "R20",
        "R31",
        "R36",
        "R41",
    ],
    "Q": [
        "B08",
        "D24",
        "D25",
        "D26",
        "D27",
        "D28",
        "D29",
        "D30",
        "D31",
        "D32",
        "D33",
        "D34",
        "D35",
        "D37",
        "D38",
        "D39",
        "D40",
        "D41",
        "D42",
        "D43",
        "Q01",
        "Q03",
        "Q04",
        "Q05",
        "R14",
        "R16",
        "R17",
        "R20",
        "R30",
    ],
    "R": [
        "G08",
        "G09",
        "G10",
        "G11",
        "G12",
        "G13",
        "G14",
        "G15",
        "G16",
        "G18",
        "G19",
        "G20",
        "G21",
        "R11",
        "R13",
        "R14",
        "R15",
        "R16",
        "R17",
        "R18",
        "R19",
        "R20",
        "R21",
        "R22",
        "R23",
        "R24",
        "R25",
        "R26",
        "R27",
        "R28",
        "R29",
        "R30",
        "R31",
        "R32",
        "R33",
        "R34",
        "R35",
        "R36",
        "R39",
        "R40",
        "R41",
        "R42",
        "R43",
        "R44",
        "R45",
    ],
    "S": ["901", "902", "D26", "H04", "H12", "H13", "H14", "H15", "S01", "S03", "S04"],
    "SIR": [
        "S09",
        "S11",
        "S13",
        "S14",
        "S15",
        "S16",
        "S17",
        "S18",
        "S19",
        "S20",
        "S21",
        "S22",
        "S23",
        "S24",
        "S25",
        "S26",
        "S27",
        "S28",
        "S29",
        "S30",
        "S31",
    ],
    "W": [
        "R01",
        "R03",
        "R04",
        "R05",
        "R06",
        "R08",
        "R09",
        "R11",
        "R13",
        "R14",
        "R15",
        "R16",
        "R17",
        "R18",
        "R19",
        "R20",
        "R21",
        "R22",
        "R23",
        "R24",
        "R25",
        "R26",
        "R27",
    ],
    "Z": [
        "G05",
        "G06",
        "J12",
        "J14",
        "J15",
        "J17",
        "J20",
        "J21",
        "J23",
        "J24",
        "J27",
        "J28",
        "J30",
        "M11",
        "M16",
        "M18",
        "M19",
        "M20",
        "M21",
        "M22",
        "M23",
    ],
}


def get_valid_trains() -> List[str]:
    """Get list of all valid train IDs"""
    return list(SUBWAY_LINES.keys())


def get_stops_for_train(train_id: str) -> List[str]:
    """Get all stop IDs served by a train line"""
    return SUBWAY_LINES.get(train_id.upper(), [])


def is_valid_train(train_id: str) -> bool:
    """Check if a train ID is valid"""
    return train_id.upper() in SUBWAY_LINES
